// Exemple d'utilisation des quaternions des fichiers quatern.sci et geouti.sci
//------------------------------------------------------------------------------
// Dans la console Scilab, faire un cd dans le répertoire où se trouvent les
// fichiers exemple.sce, quatern.sci et geouti.sci et taper :
// exec('exemple.sce', -1)
//------------------------------------------------------------------------------
p = funcprot();
funcprot(0); // pour éviter le warning de redéfinition
exec('quatern.sci',-1); // On charge les fonctions
funcprot(p);// on remet le mode de protection des fonctions à son niveau initial
disp("CALCULS EN QUATERNIONS")
disp('Rotation 30 dg autour de Oz suivie d''une rotation de 60 dg autour de Oy');
Qab1_a = quat(%pi/6, 3);
Qb1c1_b1 = quat(%pi/3, 2);
Qac1_a = Qab1_a * Qb1c1_b1;
disp(Qac1_a);

// Extraction des paramètres d'Euler
v = Qac1_a(2:4); nv = norm(v);
disp(['Rotation totale de' string(2*asin(nv)*180./%pi) 'dg autour de [' string(v'/nv) ']'])

disp("Les deux rotations effectuées dans l''ordre inverse :")
Qab2_a = Qb1c1_b1;
Qb2c2_b2 = Qab1_a;
Qac2_a = Qab2_a * Qb2c2_b2;
disp(Qac2_a);

// Extraction des paramètres d'Euler
v = Qac2_a(2:4); nv = norm(v);
disp(['Rotation totale de' string(2*asin(nv)*180./%pi) 'dg autour de [' string(v'/nv) ']'])

disp("On constate que les angles de rotation sont identiques mais que les axes sont différents")
disp("avec les composantes selon Ox de signes opposés")

// Rotation faisant passer de c1 à c2, le tout exprimé dans la base A
// Qc1c2_a*Qac1_a = Qac2_a -> Qc1c2_a = Qac2_a*Qac1_a' (en postmultipliant)
Qc1c2_a = Qac2_a*Qac1_a';
disp("Rotation faisant passer de la première à la deuxième (exprimée dans Ba):")
v_a = Qc1c2_a(2:4); nv = norm(v_a);
disp(['   Angle' string(2*asin(nv)*180./%pi) 'dg autour de [' string(v_a'/nv) ']'])

// Rotation faisant passer de c1 à c2, le tout exprimé dans la base c1 (ou c2)
// Qac1_a*Qc1c2_c1 = Qac2_a -> Qc1c2_c1 = Qac1_a'*Qac2_a (en prémultipliant)
Qc1c2_c1 = Qac1_a'*Qac2_a;
disp("Rotation faisant passer de la première à la deuxième (exprimée dans c1 ou c2):")
v_c1 = Qc1c2_c1(2:4); nv = norm(v_c1);
disp(['   Angle' string(2*asin(nv)*180./%pi) 'dg autour de [' string(v_c1'/nv) ']'])
// Vérification que c'est bien la même direction. Exprime v dans Ba pour comparer
// avec le résultat précédent
// Changement de base, vecteur v
V_a = Qac1_a*quat(v_c1)*Qac1_a'; v_a = V_a(2:4);
disp(['   direction dans Ba : [' string(v_a'/nv) ']'])
disp(['Vérification 0 ?= ' string(V_a(1))])

// Extraction des angles Aero à partir des quaternions
angs = vrot(Qac1_a.h, 'zyx');
disp(["Angles d''Euler ordre zyx pour la première rotation :" string(angs'*180/%pi)])
angs = vrot(Qac2_a.h, 'yzx');
disp(["Angles d''Euler ordre yzx pour la deuxième rotation :" string(angs'*180/%pi)])
angs = vrot(Qac2_a.h, 'zyx');
disp(["Angles d''Euler ordre zyx pour la deuxième rotation :" string(angs'*180/%pi)])
// Pour tester d'autres fonctions
angs = mat2carol(mrot(Qac2_a.h))
disp("Autre approche :")
disp(["Angles d''Euler ordre zyx pour la deuxième rotation :" string(angs'*180/%pi)])

disp("CALCULS EN MATRICES 3x3")
disp('Rotation 30 dg autour de Oz suivie d''une rotation de 60 dg autour de Oy');
Mab1_a = mrot(%pi/6, 3);
Mb1c1_b1 = mrot(%pi/3, 2);
Mac1_a = Mab1_a * Mb1c1_b1;
disp(Mac1_a);

// Extraction des paramètres d'Euler
[a, u] = vrot(Mac1_a);
disp(['Rotation totale de' string(a*180./%pi) 'dg autour de [' string(u') ']'])

disp("Les deux rotations effectuées dans l''ordre inverse :")
Mab2_a = Mb1c1_b1;
Mb2c2_b2 = Mab1_a;
Mac2_a = Mab2_a * Mb2c2_b2;
disp(Mac2_a);

// Extraction des paramètres d'Euler
[ang, u] = vrot(Mac2_a);
disp(['Rotation totale de' string(ang*180./%pi) 'dg autour de [' string(u') ']'])

// Rotation faisant passer de c1 à c2, le tout exprimé dans la base A
// Mc1c2_a*Mac1_a = Mac2_a -> Mc1c2_a = Mac2_a*Mac1_a' (en postmultipliant)
Mc1c2_a = Mac2_a*Mac1_a';
disp("Rotation faisant passer de la première à la deuxième (exprimée dans Ba):")
[ang, u_a] = vrot(Mc1c2_a);
disp(['   Angle' string(ang*180./%pi) 'dg autour de [' string(u_a') ']'])

// Rotation faisant passer de c1 à c2, le tout exprimé dans la base c1 (ou c2)
// Mac1_a*Mc1c2_c1 = Mac2_a -> Mc1c2_c1 = Mac1_a'*Mac2_a (en prémultipliant)
Mc1c2_c1 = Mac1_a'*Mac2_a;
disp("Rotation faisant passer de la première à la deuxième (exprimée dans c1 ou c2):")
[ang, u_c1] = vrot(Mc1c2_c1);
disp(['   Angle' string(ang*180./%pi) 'dg autour de [' string(u_c1') ']'])
// Vérification que c'est bien la même direction. Exprime v dans Ba pour comparer
// avec le résultat précédent
// Changement de base, vecteur v
w_a = Mac1_a * u_c1;
disp(['   direction dans Ba : [' string(w_a') ']'])
disp("On retrouve tous les résultats du calcul en quaternions")

// Extraction des angles Aero à partir des matrices
angs = vrot(Mac1_a, 'zyx');
disp(["Angles d''Euler ordre zyx pour la première rotation :" string(angs'*180/%pi)])
angs = vrot(Mac2_a, 'yzx');
disp(["Angles d''Euler ordre yzx pour la deuxième rotation :" string(angs'*180/%pi)])
angs = vrot(Mac2_a, 'zyx');
disp(["Angles d''Euler ordre zyx pour la deuxième rotation :" string(angs'*180/%pi)])
// Pour tester d'autres fonctions
angs = mat2carol(Mac2_a)
disp("Autre approche :")
disp(["Angles d''Euler ordre zyx pour la deuxième rotation :" string(angs'*180/%pi)])
